<div>
    <p>Hi {{name}},</p>
</div>
<div>
    <p>Summary of you budget request TicketId: {{ ticket_id }}</p>
    <ul>
        <li>Budget: <b>${{ budget }}</b></li>
        <li>Spending: <b>${{ used_budget }}</b></li>
        <li>OverSpend: <b>${{ remain_budget }}</b></li>
    </ul>
    <p>If you want to keep using this allocation, please extend the budget:</p>
    <ul>
        <li>Click <a href="{{portal}}/extend_budget">here</a> to open Budget extend page</li>
        <li>Fill the details, and select the TicketId: {{ ticket_id }}</li>
        <li>Enter the amount and description of the why you need extension.</li>
        <li>Then click on submit</li>
    </ul>
    <p>If you don't need to use this allocation anymore, you can ignore this email. Please don't forgot to terminate the resources,<br/>
    You won't receive additional emails <b>as long as the allocation cost does not increase</b>.</p>
</div>
{{footer}}
